import os
import pandas as pd

def to_dataframe(data: dict | list[dict]):
    """Output as Dataframe"""

    if isinstance(data, list) and not all(isinstance(item, dict) for item in data):
        raise TypeError("All items in the list must be dictionaries")
    elif not isinstance(data, (dict, list)):
        raise TypeError("data must be a dict or a list of dicts")
    
    if isinstance(data, dict):
        data = [data]      

    return pd.DataFrame(data)

def save_to_csv(data, filename: str = "output.csv", path: str = None):
    """Save a list of dicts to CSV"""

    if path is None:
        path = os.getcwd()
    else:
        path = os.path.abspath(path)

    full_path = os.path.join(path, filename)

    df = to_dataframe(data)

    df.to_csv(full_path, index=False)

    print(f"Saved CSV to: {full_path}")

    return True

def save_to_excel(data: dict | list[dict], filename: str = "output.xlsx", path: str = None):
    """Save data to an Excel spreadsheet."""
    
    if path is None:
        path = os.getcwd()
    else:
        path = os.path.abspath(path)

    full_path = os.path.join(path, filename)
    df = to_dataframe(data)
    df.to_excel(full_path, index=False)
    
    print(f"Saved Excel Document to: {full_path}")
    
    return True






from financegy import *
import pandas as pd

# def test_get_securities():
#     result = get_securities()
#     assert isinstance(result, list)

# def test_get_security_by_symbol():
#     result = get_security_by_symbol(symbol="ddl")    
#     assert isinstance(result, (str, type(None)))

# def test_get_recent_trade():
#     result = get_recent_trade(symbol="ddl")
#     assert isinstance(result, (dict, type(None)))

# def test_get_security_recent_year():
#     result = get_security_recent_year(symbol="RBL")
#     assert isinstance(result, (list, type(None)))
    
# def test_get_session_trades():
#     result = get_session_trades(session="1135")
#     assert isinstance(result, (list, type(None)))

# def test_get_security_session_trade():
#     result = get_security_session_trade(symbol="ddl", session="1136")
#     assert isinstance(result, (dict, type(None)))

# def test_get_trades_for_year():
#     result = get_trades_for_year(symbol="ddl", year="2020")
#     assert isinstance(result, (list, type(None)))

# def test_get_historical_trades():
#     result = get_historical_trades(symbol="ddl", start_date="06/2020", end_date="01/2022")
#     assert isinstance(result, (list, type(None)))

# def test_search_securities():
#     result = search_securities(query="ddl")
#     assert isinstance(result, list)
    
# def test_to_dataframe():
#     result = to_dataframe(get_securities())
#     assert isinstance(result, pd.DataFrame)

# def test_save_to_csv():
#     result = get_securities()
#     save_to_csv(result)
#     assert isinstance(result, list)

# def test_save_excel():
#     result = get_securities()
#     save_to_excel(result)
#     assert isinstance(result, list)


# def test_clear_cache():
#     result = clear_cache()
#     assert isinstance(result, bool)


import os, json, hashlib
from datetime import datetime, timedelta
import shutil

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
CACHE_DIR = os.path.join(SCRIPT_DIR, "cache")

def make_cache_key(func_name, *args, **kwargs):
    """Create a unique hash for the given function call."""

    key_data = {
        "func": func_name,
        "args": args,
        "kwargs": kwargs
    }

    key_string = json.dumps(key_data, sort_keys=True, default=str)
    hashed = hashlib.md5(key_string.encode()).hexdigest()

    return f"{func_name}_{hashed}.json"

def load_cache(func_name, *args, max_age_days=7, **kwargs):
    os.makedirs(CACHE_DIR, exist_ok=True)
    cache_file = make_cache_key(func_name, *args, **kwargs)
    cache_path = os.path.join(CACHE_DIR, cache_file)

    if not os.path.exists(cache_path):
        return None

    with open(cache_path, "r") as f:
        data = json.load(f)

    timestamp = datetime.fromisoformat(data["timestamp"])
    if datetime.now() - timestamp > timedelta(days=max_age_days):
        return None

    return data["value"]

def save_cache(func_name, value, *args, **kwargs):
    os.makedirs(CACHE_DIR, exist_ok=True)
    cache_file = make_cache_key(func_name, *args, **kwargs)
    cache_path = os.path.join(CACHE_DIR, cache_file)

    with open(cache_path, "w") as f:
        json.dump({
            "timestamp": datetime.now().isoformat(),
            "value": value
        }, f)

def clear_cache():
    """Completely clears the FinanceGY cache directory."""
    if not os.path.exists(CACHE_DIR):
        print("No cache directory found.")
        return False

    try:
        shutil.rmtree(CACHE_DIR)
        print("Cache cleared successfully.")
        return True
        
    except Exception as e:
        print(f"Failed to clear cache: {e}")